/*

 #        ####    ####   #    #           ####
 #       #    #  #    #  #   #           #    #
 #       #    #  #       ####            #
 #       #    #  #       #  #     ###    #
 #       #    #  #    #  #   #    ###    #    #
 ######   ####    ####   #    #   ###     ####

	Lock file handling.
*/

#include "config.h"
#include "portable.h"
#include <errno.h>
#include <sys/types.h>
#include <sys/stat.h>

#ifdef I_FCNTL
#include <fcntl.h>
#else
#include <sys/fcntl.h>
#endif
#ifdef I_SYSFILE
#include <sys/file.h>
#endif

#define MAX_STRING	2048		/* Max string length */
#define MAX_TIME	3600		/* One hour */

private char lockfile[MAX_STRING];		/* Location of lock file */
private int locked = 0;					/* Did we lock successfully? */

private void check_lock();				/* Make sure lockfile is not too old */

extern int errno;						/* System error status */
extern Time_t time();					/* Current time */

public int filter_lock(dir)
char *dir;						/* Where lockfile should be written */
{
	/* Note: this locking is not completly safe w.r.t. race conditions, but the
	 * mailagent will do its own locking checks in a rather safe way.
	 * Return 0 if locking succeeds, -1 otherwise.
	 */

	int fd;

	sprintf(lockfile, "%s/filter.lock", dir);
	check_lock(lockfile);
	if (-1 == (fd = open(lockfile, O_CREAT | O_EXCL, 0))) {
		if (errno != EEXIST)
			add_log(1, "SYSERR open: %m (%e)");
		return -1;
	}
	locked = 1;					/* We did lock successfully */
	close(fd);					/* Close dummy file descriptor */

	return 0;
}

public void release_lock()
{
	if (locked && -1 == unlink(lockfile)) {
		add_log(1, "SYSERR unlink: %m (%e)");
		add_log(4, "WARNING could not remove lockfile %s", lockfile);
	}
	locked = 0;
}

public int is_locked()
{
	return locked;			/* Do we have a lock file active or not? */
}

private void check_lock(file)
char *file;
{
	/* Make sure the lock file is not older than MAX_TIME seconds, otherwise
	 * unlink it (something must have gone wrong).
	 */

	struct stat buf;

	if (-1 == stat(file, &buf)) {		/* Stat failed */
		if (errno == ENOENT)			/* File does not exist */
			return;
		add_log(1, "SYSERR stat: %m (%e)");
		add_log(2, "could not check lockfile %s", file);
		return;
	}

	if ((buf.st_mtime - time((Time_t *) 0)) > MAX_TIME) {
		if (-1 == unlink(lockfile)) {
			add_log(1, "SYSERR unlink: %m (%e)");
			add_log(4, "WARNING could not remove old lock %s", lockfile);
		} else
			add_log(6, "UNLOCKED filter (lock older than 1 hour)");
	} else
		add_log(16, "lockfile for filter is recent");
}

