;# $Id: lexical.pl,v 2.9.1.2 92/11/01 15:50:52 ram Exp $
;#
;#  Copyright (c) 1992, Raphael Manfredi
;#
;#  You may redistribute only under the terms of the GNU General Public
;#  Licence as specified in the README file that comes with dist.
;#
;# $Log:	lexical.pl,v $
;# Revision 2.9.1.2  92/11/01  15:50:52  ram
;# patch11: fixed English typo
;# 
;# Revision 2.9.1.1  92/08/02  16:11:18  ram
;# patch2: added support for negated selectors
;# 
;# Revision 2.9  92/07/14  16:50:10  ram
;# 3.0 beta baseline.
;# 
;#
#
# Lexical parsing of the rules
#

# The following subroutine is called whenever a new rule input is needed.
# It returns that new line or a null string if end of file has been reached.
sub read_filerule {
	<RULES>;					# Read a new line from file
}

# The following subroutine is called in place of read_rule when rules are
# coming from the command line via @Linerules.
sub read_linerule {
	shift(@Linerules);			# Read a new line from array
}

# Assemble a whole rule in one line and return it. The end of a line is
# marked by a ';' at the end of an input line.
sub get_line {
	local($result) = "";		# what will be returned
	local($in_braces) = 0;		# are we inside braces ?
	for (;;) {
		$_ = &read_rule;		# new rule line (pseudo from compile_rules)
		last if $_ eq '';		# end of file reached
		s/\n$//;				# don't use chop in case we read from array
		next if /^\s*#/;		# skip comments
		s/\s\s+/ /;				# reduce white spaces
		$result .= $_;
		# Very simple braces handling
		/.*{/ && ($in_braces = 1);
		if ($in_braces) {
			/.*}/ && ($in_braces = 0);
		}
		last if !$in_braces && /;\s*$/;
	}
	$result;
}

# Get optional mode (e.g. <TEST>) at the beginning of the line and return
# it, or ALL if none was present.
sub get_mode {
	local(*line) = shift(@_);	# edited in place
	local($_) = $line;			# make a copy of original
	local($mode) = "ALL";		# default mode
	s/^<([\s\w,]+)>// && ($mode = $1);
	$mode =~ s/\s//g;			# no spaces in returned mode
	$line = $_;					# eventually updates the line
	$mode;
}

# A selector is either a script or a list of header fields ending with a ':'.
sub get_selector {
	local(*line) = shift(@_);	# edited in place
	local($_) = $line;			# make a copy of original
	local($selector) = "";
	s/^\s*,//;					# remove rule separator
	if (/^\s*\[\[/) {			# detected a script form
		$selector = 'script:';
	} else {
		s/^\s*([^\/,{\n]*:)// && ($selector = $1);
	}
	$line = $_;					# eventually updates the line
	$selector;
}

# A pattern if either a single word (with no white space) or something
# starting with a / and ending with an un-escaped / followed by some optional
# modifiers.
# Patterns may be preceded by a single '!' to negate the matching value.
sub get_pattern {
	local(*line) = shift(@_);		# edited in place
	local($_) = $line;				# make a copy of original
	local($pattern) = "";			# the recognized pattern
	local($buffer) = "";			# the buffer used for parsing
	local($not) = '';				# shall boolean value be negated?
	s|^\s*||;						# remove leading spaces
	s/^!// && ($not = '!');			# A leading '!' inverts matching status
	if (s|^\[\[([^{]*)\]\]||) {		# pattern is a script
		$pattern = $1;				# get the whole script
	} elsif (s|^/||) {				# pattern starts with a /
		$pattern = "/";				# record the /
		while (s|([^/]*/)||) {		# while there is something before a /
			$buffer = $1;			# save what we've been reading
			$pattern .= $1;
			last unless $buffer =~ m|\\/$|;	# finished unless / is escaped
		}
		s/^(\w+)// && ($pattern .= $1);		# add optional modifiers
	} else {								# pattern does not start with a /
		s/([^\s,;{]*)// && ($pattern = $1);	# grab all until next delimiter
	}
	$line = $_;					# eventually updates the line
	$pattern =~ s/\s+$//;		# remove trailing spaces
	if ($not && !$pattern) {
		&add_log("ERROR discarding '!' not followed by pattern") if $loglvl;
	} else {
		$pattern = $not . $pattern;
	}
	$pattern;
}

sub get_action {
	local(*line) = shift(@_);	# edited in place
	local($_) = $line;			# make a copy of original
	local($action) = "";
	if (s/^\s*{([^}]*)}//) {
		$action = $1;
	} else {
		unless (/\{.*\}/) {		# trash line if no { action } is present
			&add_log("ERROR expected action, discarded '$_'") if $loglvl;
			$_ = '';
		}
	}
	$line = $_;					# eventually updates the line
	$action =~ s/\s+$//;		# remove trailing spaces
	$action;
}

