;# $Id: macros.pl,v 2.9.1.2 92/08/26 13:16:14 ram Exp $
;#
;#  Copyright (c) 1992, Raphael Manfredi
;#
;#  You may redistribute only under the terms of the GNU General Public
;#  Licence as specified in the README file that comes with dist.
;#
;# $Log:	macros.pl,v $
;# Revision 2.9.1.2  92/08/26  13:16:14  ram
;# patch8: added support for external variables (persistent)
;# 
;# Revision 2.9.1.1  92/07/25  12:40:37  ram
;# patch1: now discards comments in Reply-To field (macro %r)
;# 
;# Revision 2.9  92/07/14  16:50:16  ram
;# 3.0 beta baseline.
;# 
;# 
# Macros:
# %%     A real percent sign
# %D     Day of the week (0-6)
# %H     Host name (name of the machine on which the mailagent runs)
# %L     Length of the message in bytes (without header)
# %N     Full name of sender (login name if none)
# %R     Subject of orginal message with leading Re: suppressed
# %S     Re: subject of original message
# %T     Time of last modification on mailed file (value taken from $macro_T)
# %U     Full name of the user
# %_     A white space
# %#reg  Value of user-defined variable 'reg'
# %&     List of selectors which incurred match (among regular-expression ones) 
# %~     A null character
# %1     Value of the corresponding backreference (limited to 99 per rule)
# %d     Day of the month (01-31)
# %f     Contents of the "From:" line, something like %N <%r> or %r (%N)
# %h     Hour of the day (00-23)
# %i     Message ID if available
# %l     Number of lines in the message
# %m     Month of the year (01-12)
# %n     Lower-case login name of sender
# %o     Organization (where mailagent runs)
# %r     Return address of message
# %s     Subject of original message
# %t     Current hour and minute (in HH:MM format)
# %u     Login name of the user
# %y     Year (last two digits)
# %[To]  Value of the field in header (here To:)

# Macros substitutions (in-place)
sub macros_subst {
	local(*str) = shift(@_);			# The string
	local($_) = $str;					# Work on a copy
	return unless /%/;					# Return immediately if no macros

	local($sender);							# The from field
	local(@from);							# The rfc-822 parsed from line
	$sender = $Header{'From'};				# Header-derived From address
	@from = &parse_address($sender);		# Get (address, comment)
	local($login) = &login_name($from[0]);	# Keep only login name
	local($fullname) = $from[1];			# The comment part of address
	$fullname = $login unless $fullname;	# Use login name if no comment part
	local($reply_to) = $Header{'Reply-To'}; # Return path derived
	local($subject) = $Header{'Subject'};	# Original subject header
	$subject =~ s/^\s*Re:\s*(.*)/$1/;		# Strip off leading Re:
	$subject = "<empty subject>" unless $subject;
	$reply_to = (&parse_address($reply_to))[0];	# Keep only e-mail address

	# Time computations
	local($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) =
			localtime(time);
	$mon = sprintf("%.2d", $mon + 1);
	$mday = sprintf("%.2d", $mday);
	local($timenow) = sprintf("%.2d:%.2d", $hour, $min);
	$hour = sprintf("%.2d", $hour);

	# The following dummy block is here only to force perl interpreting
	# the $ variables in the substitutions correctly...
	if (0) {
		$Header{'a'} = 'a';
		$Variable{'a'} = 'a';
		$Backref[0] = 0;
	}

	s/%%/##pr##/g;						# Protect double percent signs
	s/%/#%%!/g;							# Make sure substitutions do not add %
	s/#%%!d/$mday/g;					# Day of the month (01-31)
	s/#%%!D/$wday/g;					# Day of the week (0-6)
	s/#%%!f/$Header{'From'}/g;			# The "From:" line
	s/#%%!h/$hour/g;					# Hour of the day (00-23)
	s/#%%!i/$Header{'Message-Id'}/g;	# Message-Id (null string if none)
	s/#%%!l/$Header{'Lines'}/g;			# Number if lines in message
	s/#%%!L/$Header{'Length'}/g;		# Length of message, in bytes
	s/#%%!m/$mon/g;						# Month of the year
	s/#%%!n/$login/g;					# Lower-cased login name of sender
	s/#%%!N/$fullname/g;				# Full name of sender (login if none)
	s/#%%!o/$orgname/g;					# Organization name
	s/#%%!r/$reply_to/g;				# Return path of message
	s/#%%!R/$subject/g;					# Subject with leading Re: suppressed
	s/#%%!s/$Header{'Subject'}/g;		# Subject of message
	s/#%%!S/Re: $Header{'Subject'}/g;	# Re: subject of original message
	s/#%%!t/$timenow/g;					# Current time HH:MM
	s/#%%!T/$macro_T/g;					# Time of last modification on file
	s/#%%!u/$cf'user/go;				# User login name (does not change)
	s/#%%!U/$cf'name/go;				# User's name (does not change)
	s/#%%!y/$year/g;					# Year (last two digits)
	s/#%%!_/ /g;						# A white space
	s/#%%!~//g;							# A null character
	s/#%%!&/$macro_ampersand/g;			# List of matched generic selectors
	s/#%%!(\d\d?)/$Backref[$1 - 1]/g;	# A pattern matching backreference
	s/#%%!#:(\w+)/&extern'val($1)/eg;	# A persistent user-defined variable
	s/#%%!#(\w+)/$Variable{$1}/g;		# A user-defined variable
	s/#%%!\[([\w-]+)\]/$Header{$1}/g;	# The %[Field] macro
	s/#%%!/%/g;							# Any remaining percent is kept
	s/##pr##/%/g;						# A double percent expands to %
	$str = $_;							# Update string in-place
}

