;# $Id: parse.pl,v 2.9.1.1 92/08/26 13:17:47 ram Exp $
;#
;#  Copyright (c) 1992, Raphael Manfredi
;#
;#  You may redistribute only under the terms of the GNU General Public
;#  Licence as specified in the README file that comes with dist.
;#
;# $Log:	parse.pl,v $
;# Revision 2.9.1.1  92/08/26  13:17:47  ram
;# patch8: created by extraction from analyze.pl
;# patch8: parsing can now be done on header only
;# 
;# 
#
# Parsing mail
#

# Parse the mail and fill-in the Header associative array. The special entries
# All, Body and Head respectively hold the whole message, the body and the
# header of the message.
sub parse_mail {
	local($file_name) = shift(@_);	# Where mail is stored ("" for stdin)
	local($head_only) = shift(@_);	# Optional parameter: parse only header
	local($last_header) = "";		# Name of last header (for continuations)
	local($first_from) = "";		# The first From line in mails
	local($lines) = 0;				# Number of lines in the body
	local($length) = 0;				# Length of body, in bytes
	local($last_was_nl) = 1;		# True when last line was a '\n' (1 for EOH)
	local($fd) = STDIN;				# Where does the mail come from ?
	local($value);					# Value of current field line
	local($_);
	undef %Header;					# Reset the all structure holding message

	if ($file_name ne '') {			# Mail spooled in a file
		unless(open(MAIL, $file_name)) {
			&add_log("ERROR cannot open $file_name: $!");
			return;
		}
		$fd = MAIL;
	}
	$Userpath = "";					# Reset path from possible previous @PATH 

	# Pre-extend 'All', 'Body' and 'Head'
	$Header{'All'} = ' ' x 5000;
	$Header{'Body'} = ' ' x 4500;
	$Header{'Head'} = ' ' x 500;
	$Header{'All'} = '';
	$Header{'Body'} = '';
	$Header{'Head'} = '';

	do add_log ("parsing mail") if $loglvl > 18;
	while (<$fd>) {
		$Header{'All'} .= $_;
		if (1../^$/) {						# EOH is a blank line
			next if /^$/;					# Skip EOH marker
			$Header{'Head'} .= $_;			# Record line in header

			if (/^\s/) {					# It is a continuation line
				s/^\s+/ /;					# Swallow multiple spaces
				chop;						# Remove final new-line
				$Header{$last_header} .= "\n$_" if $last_header ne '';
				do add_log("WARNING bad continuation in header, line $.")
					if $last_header eq '' && $loglvl > 4;
			} elsif (/^([\w-]+):\s*(.*)/) {	# We found a new header
				# Guarantee only one From: header line. If multiple From: are
				# found, keep the last one.
				# Multiple headers like 'Received' are separated by a new-
				# line character. All headers end on a non new-line.
				# Case is normalized before recording, so apparently-to will
				# be recorded as Apparently-To but header is not changed.
				$value = $2;				# Bug in perl 4.0 PL19
				$last_header = &header'normalize($1);	# Normalize case
				if ($last_header eq 'From' && defined $Header{$last_header}) {
					$Header{$last_header} = $value;
					do add_log("WARNING duplicate From in header, line $.")
						if $loglvl > 4;
				} elsif ($Header{$last_header} ne '') {
					$Header{$last_header} .= "\n$value";
				} else {
					$Header{$last_header} .= $value;
				}
			} elsif (/^From\s+(\S+)/) {		# The very first From line
				$first_from = $1;
			}

		} else {
			last if $head_only;		# Stop parsing if only header wanted
			$lines++;								# One more line in body
			$length += length($_);					# Update length of message
			s/^From(\s)/>From$1/ if $last_was_nl;	# Escape From keyword
			$last_was_nl = /^$/;					# Keep track of single '\n'
			$Header{'Body'} .= $_;
			chop;
			# Deal with builtin commands
			if (s/^@(\w+)\s*//) {					# A builtin command ?
				local($subroutine) = $Builtin{$1};
				&$subroutine($_) if $subroutine;
			}
		}
	}
	close MAIL if $file_name ne '';
	&header_check($first_from, $lines);	# Sanity checks
}

# Now do some sanity checks:
# - if there is no From: header, fill it in with the first From
# - if there is no To: but an Apparently-To:, copy it also as a To:
#
# We guarantee the following header entries:
#   From:         the value of the From field
#   To:           to whom the mail was sent
#   Lines:        number of lines in the message
#   Length:       number of bytes in the message
#   Reply-To:     the address we may use to reply
#   Sender:       the actual sender, even if same as From

sub header_check {
	local($first_from, $lines) = @_;	# First From line, number of lines
	unless (defined $Header{'From'}) {
		&add_log("WARNING no From: field, assuming $first_from") if $loglvl > 4;
		$Header{'From'} = $first_from;
	}

	# There is usually one Apparently-To line per address. Remove all new lines
	# in the header line and replace them with ','.
	$* = 1;
	$Header{'Apparently-To'} =~ s/\n/,/g;	# Remove new-lines
	$* = 0;
	$Header{'Apparently-To'} =~ s/,$/\n/;	# Restore last new-line

	# If no To: field, then maybe there is an Apparently-To: instead. If so,
	# make them identical. Otherwise, assume the mail was directed to the user.
	if (!$Header{'To'} && $Header{'Apparently-To'}) {
		$Header{'To'} = $Header{'Apparently-To'};
	}
	unless ($Header{'To'}) {
		do add_log("WARNING no To: field, assuming $cf'user") if $loglvl > 4;
		$Header{'To'} = $cf'user;
	}

	# Set number of lines in body, unless there is already a Lines:
	# header in which case we trust it. Same for Length.
	$Header{'Lines'} = $lines unless defined($Header{'Lines'});
	$Header{'Length'} = $length unless defined($Header{'Length'});

	# If there is no Reply-To: line, then take the return path, if any.
	# Otherwise use the address found in From.
	if (!$Header{'Reply-To'}) {
		local($tmp) = $Header{'Return-Path'};
		$tmp =~ /<(.*)>/ && ($tmp = $1);		# Remove the <> in path
		$Header{'Reply-To'} = $tmp if $tmp;
		$Header{'Reply-To'} = (&parse_address($Header{'From'}))[0] unless $tmp;
	}

	# Unless there is already a sender line, fake one using From field
	if (!$Header{'Sender'}) {
		$Header{'Sender'} = $first_from;
	}
}

