;# $Id: read_conf.pl,v 2.9.1.2 92/08/12 21:33:29 ram Exp $
;#
;#  Copyright (c) 1991, 1992, Raphael Manfredi
;#
;#  You may redistribute only under the terms of the GNU General Public
;#  Licence as specified in the README file that comes with dist.
;#
;# $Log:	read_conf.pl,v $
;# Revision 2.9.1.2  92/08/12  21:33:29  ram
;# patch6: perform security checks after config file parsing
;# 
;# Revision 2.9.1.1  92/07/25  12:42:08  ram
;# patch1: configuration errors now reported on stderr
;# patch1: first syntax error also logged for further diagnostic
;# 
;# Revision 2.9  92/07/14  16:50:40  ram
;# 3.0 beta baseline.
;# 
;#
package cf;

# This package is responsible for keeping track of the configuration variables.

# Read configuration file (usually in ~/.mailagent)
sub main'read_config {
	local($file) = @_;				# where config file is located
	local($_);
	$file = '~/.mailagent' unless $file;
	local($myhome) = $ENV{'HOME'};	# must be correctly set by filter
	$file =~ s/~/$myhome/;			# ~ substitution
	local($main'config) = $file;	# Save it: could be modified by config
	open(CONFIG, "$file") ||
		&'fatal("can't open config file $file");
	local($config) = ' ' x 2000;	# pre-extend to avoid realloc()
	$config = '';
	while (<CONFIG>) {
		next if /^[ \t]*#/;			# skip comments
		next if /^[ \t]*\n/;		# skip empy lines
		$config .= $_;
	}
	&parse($config) || &'fatal('bad configuration');
	close CONFIG;

	# Security checks, pending of those performed by the C filter. They are
	# somewhat necessary, even though the mailagent does not run setuid
	# (because anybody may activate the mailagent for any user by sending him
	# a mail, and world writable configuration files makes the task too easy
	# for a potential hacker). The tests are performed once the configuration
	# file has been parsed, so logging of fatal errors may occur.

	local($ST_MODE) =  2 + $[;		# Field st_mode from inode structure
	local($S_IWOTH) = 02;			# Writable by world (no .ph files here)

	-O "$'config" || &'fatal("you do not own config file $'config");
	local($st_mode) = (stat($'config))[$ST_MODE];
	($st_mode & $S_IWOTH) && &'fatal("config file $'config is world writable!");

	return unless -f "$rules";		# No rule file

	-O "$rules" || &'fatal("you do not own rule file $rules");
	$st_mode = (stat($rules))[$ST_MODE];
	($st_mode & $S_IWOTH) && &'fatal("rule file $rules is world writable!");
}

# Parse config file held in variable and return 1 if ok, 0 for errors
sub parse {
	local($config) = @_;
	local($eval) = ' ' x 1000;		# Pre-extend
	local($myhome) = $ENV{'HOME'};	# must be correctly set by filter
	local($var, $value);
	local($_);
	$eval = '';
	foreach (split(/\n/, $config)) {
		if (/^[ \t]*([^ \t\n:\/]*)[ \t]*:[ \t]*([^#\n]*)/) {
			$var = $1;
			$value = $2;
			$value =~ s/\s*$//;						# remove trailing spaces
			$eval .= "\$$var = \"$value\";\n";
			$eval .= "\$$var =~ s|~|$myhome|g;\n";	# ~ substitution
		}
	}
	eval $eval;			# evaluate configuration parameters within package

	if ($@ ne '') {				# Parsing error detected
		local($error) = $@;		# Logged error
		local($*) = 1;
		$error = (split(/\n/, $error))[0];		# Keep only first line
		# Dump error message on stderr, as well as faulty configuration file.
		# The original is restored out of the perl form to avoid surprise.
		$eval =~ s/^\$.* =~ s\|~\|.*\n//g;		# Remove added ~ substitutions
		$eval =~ s/^\$//g;						# Remove leading '$'
		$eval =~ s/ = "(.*)";/: $1/g;			# Keep only variable value
		chop($eval);
		print STDERR <<EOM;
**** Syntax error in configuration:
$error

---- Begin of Faulty Configuration
$eval
---- End of Faulty Configuration

EOM
		&'add_log("syntax error in configuration: $error") if $'loglvl > 1;
		return 0;
	}

	# Define the mailagent parameters from those in config file
	$logfile = $logdir . "/$log";
	$seqfile = $spool . "/$seq";
	$hashdir = $spool . "/$hash";
	$main'loglvl = int($level);		# This one is visible in the main package
	$main'track_all = 1 if $track =~ /on/i;		# Option -t set by config

	1;		# Ok
}

package main;

