;# $Id: rfc822.pl,v 2.9.1.1 92/11/01 15:51:46 ram Exp $
;#
;#  Copyright (c) 1992, Raphael Manfredi
;#
;#  You may redistribute only under the terms of the GNU General Public
;#  Licence as specified in the README file that comes with dist.
;#
;# $Log:	rfc822.pl,v $
;# Revision 2.9.1.1  92/11/01  15:51:46  ram
;# patch11: allows _ as separator in names (as in First_Last)
;# 
;# Revision 2.9  92/07/14  16:50:42  ram
;# 3.0 beta baseline.
;# 
;#
;# The following routines do some parsing on RFC822 headers (such as the
;# ones provided by sendmail).
;#
# Parse an address and returns (internet, comment)
# Examples:
#    ram@eiffel.com (Raphael Manfredi)  -> (ram@eiffel.com, Raphael Manfredi)
#    Raphael Manfredi <ram@eiffel.com>  -> (ram@eiffel.com, Raphael Manfredi)
sub parse_address {
	local($_) = shift(@_);		# The address to be parsed
	local($comment);
	local($internet);
	if (/^\s*(\S+)\s+\((.*)\)/) {		# address (comment) 
		($1, $2);
	} elsif (/^\s*(.*)\s+<(\S+)>/) {	# comment <address>
		$comment = $1;
		$internet = $2;
		$comment =~ s/^"(.*)"/$1/;		# "comment" -> comment
		($internet, $comment);
	} elsif (/^\s*<(\S+)>/) {			# <address>
		($1, "");
	} else {							# address
		s/^\s+//;
		($_, "");
	}
}

# Parses an internet address and returns the login name of the sender
sub login_name {
	local($_) = shift(@_);				# The internet address
	if (s/^"(\S+)"@\S+/$1/) {			# "user@domain"@other
		do login_name($_);				# parse user@domain
	} elsif (s/^(\S+)@\S+/$1/) {		# user@domain.name
		do login_name($_);				# parse user
	} elsif (s/^(\S+)%\S+/$1/) {		# user%domain.name
		do login_name($_);				# parse user
	} elsif (s/^\S+!(\S+)/$1/) {		# ...!backbone!user
		do last_name($_);				# user can only be a simple name
	} else {							# everything else must be a single name
		do last_name($_);				# keep only last name
	}
}

# Extract last name from a login name like First_name.Last_name and put it
# in lowercase. Hence, Raphael.Manfredi will become manfredi.
sub last_name {
	local($_) = shift(@_);			# The sender's login name
	s/.*\.(\w+)/$1/;				# Keep only the last name (. separation)
	s/.*_(\w+)/$1/;					# Same as above (_ separation)
	tr/A-Z/a-z/;					# And lowercase it
	$_;
}

