;# $Id: runcmd.pl,v 2.9.1.2 92/08/26 13:18:01 ram Exp $
;#
;#  Copyright (c) 1992, Raphael Manfredi
;#
;#  You may redistribute only under the terms of the GNU General Public
;#  Licence as specified in the README file that comes with dist.
;#
;# $Log:	runcmd.pl,v $
;# Revision 2.9.1.2  92/08/26  13:18:01  ram
;# patch8: new PERL command
;# 
;# Revision 2.9.1.1  92/08/02  16:14:10  ram
;# patch2: added support for escaping of ';' and backslash
;# patch2: new %Nostatus table records actions with no exiting status
;# patch2: the status of the last command is now recorded in lastcmd
;# 
;# Revision 2.9  92/07/14  16:50:46  ram
;# 3.0 beta baseline.
;# 
;# 
# Executing the action enclose in braces. The current working mode 'wmode' is
# a local variable defined in analyze_mail. But this variable is visible when
# 'xeqte' is called from within it. Thanks perl.

# The following commands are available (case is irrelevent):
#  ABORT                    Aborts filtering right away
#  ANNOTATE field <value>   Annotation in header a la MH
#  ASSIGN var <value>       Assign value to the user-defined variable
#  BACK <cmd>               Execute <cmd> and eval its output
#  BEGIN state              Enter in a new state for analysis
#  BOUNCE address(es)       As FORWARD but leave header intact
#  DELETE                   Trash the mail away
#  FEED program             Same as PASS, but the whole message is given
#  FORWARD address(es)      Forwards mail to specified addresses
#  GIVE program             Give the body of the message to a program
#  KEEP header(s)           Lists the header fields we want to keep
#  LEAVE                    Leave mail in incomming mailbox
#  MESSAGE vacation         Sends a vacation-like message back
#  NOP                      No operation (useful only with ONCE)
#  NOTIFY address message   Notifies address with a given message
#  ONCE (period) <cmd>      Executes any other single command once per period
#  PASS program             Pass body to program and get new body back
#  PERL script              Run script to perform some filtering actions
#  PIPE program             Pipes message to program
#  POST newsgroup(s)        Post message on specified newsgroups
#  PROCESS                  The mailagent processes the commands in body
#  PURIFY program           Feed header to program and get new header back
#  QUEUE                    Queue mail (counts as save if successful)
#  RECORD                   Record message and REJECT in seen mode if present
#  REJECT                   Abort execution and continue analysis
#  RESTART                  Abort execution and restart analysis from scratch
#  RESYNC                   Resynchronize header (useful only with FEED)
#  RUN program              Run the specified program
#  SAVE folder              Saves mail in folder for delayed reading
#  SELECT (when) <cmd>      Run command only within certain time period
#  SPLIT folder             Split digest message into folder
#  STORE folder             Same as SAVE folder; LEAVE
#  STRIP header(s)          Removes the lines from the message's header
#  SUBST var //             Apply a substitution on variable
#  TR var //                Apply a translation on variable
#  UNIQUE                   Delete message if already in history and REJECT
#  VACATION on/off			Allow/disallow vacation messages
#  WRITE folder             Writes mail in folder (replaces, does not append)

# Split the commands and execute them. This function is the main entry point
# for nesting level (e.g. execution of commands from BACK are driven by xeqte).
# We wish to keep track of the execution status of the last command, as does
# the shell with its $? variable. This is done by $lastcmd.
sub xeqte {
	local($line) = shift(@_);		# Commands to execute
	local(@cmd);					# The commands to be ran
	local($status) = $FT_CONT;		# Status returned by run_command
	local($lastcmd) = 0;			# Failure status from last command
	local($_);

	# Normally, a ';' separates each action. However, an escaped one as in \;
	# must not be taken into account. We also need to escape a single \, in
	# case we want a \ followed by a ; grr...
	$line =~ s/\\\\/\02/g;			# \\ -> ^B
	$line =~ s/\\;/\01/g;			# \; -> ^A
	@cmd = split(/;/, $line);		# Put all commands in an array
	foreach (@cmd) {				# Now restore orginal escaped sequences
		s/\01/;/g;					# ^A -> ;
		s/\02/\\/g;					# ^B -> \
	}

	# Now run each command in turn
	foreach $cmd (@cmd) {
		$status = &run_command($cmd);
		last unless $status == $FT_CONT;
	}

	# Remap $FT_ABORT on $FT_CONT. In effect, we just skipped the remaining
	# commands on the line and act as if they had been executed. This indeed
	# achieves the ABORT command.
	$status = $FT_CONT if $status == $FT_ABORT;
	$status;
}

# Executes a filter command and return continuing status:
#  FT_CONT to continue
#  FT_REJECT if a reject was found
#  FT_RESTART if a restart was found
#  FT_ABORT if an abort was found
sub run_command {
	local($cmd) = @_;				# Command to be run (passed to subroutines)
	local($cmd_name);				# Command name
	local($cont) = $FT_CONT;		# Continue by default
	local($mfile) = $file_name =~ m|.*/(.*)|;	# Basename of mail file
	$mfile = $file_name unless $mfile;			# There was no / in name
	$mfile = '<stdin>' unless $mfile;			# No $file_name if from STDIN
	do macros_subst(*cmd);			# Macros substitutions
	$cmd =~ s/^\s*//;				# Remove leading spaces
	$cmd =~ s/\s*$//;				# And trailing ones
	return $cont unless $cmd;		# Ignore null instructions
	($cmd_name) = $cmd =~ /^(\w+)/;
	$cmd_name =~ tr/a-z/A-Z/;		# In uppercase from now on
	# In the special mode _SEEN_, only a restricted set of action are allowed
	if ($wmode eq '_SEEN_') {
		if ($Rfilter{$cmd_name}) {
			do add_log("WARNING command $cmd_name not allowed") if $loglvl > 5;
			return $cont;
		}
	}
	do add_log("XEQ ($cmd)") if $loglvl > 10;
	print ">> $cmd\n" if $track_all;		# Option -t
	local($routine) = $Filter{$cmd_name};
	# Unknown commands default to LEAVE if no save have ever been done.
	# Otherwise, they are simply ignored.
	unless ($routine) {
		local($what) = 'defaults to LEAVE';
		$what = 'ignored' if $ever_saved;
		do add_log("ERROR unknown command $cmd_name ($what)")
			if $loglvl > 1;
		$routine = $Filter{'LEAVE'};		# Default action
		return $cont if $ever_saved;		# Command ignored
	}
	local($failed) = eval("&$routine");		# Eval traps all fatal errors
	$failed = 1 if &eval_error;				# Make sure eval worked

	# If command does not belong to the set of those who do not modify the
	# last execution status recorded, then update $lastcmd with the failure
	# status.
	$lastcmd = $failed unless $Nostatus{$cmd_name};

	# Update statistics
	unless ($failed) {
		&s_action($cmd_name, $wmode);
	} else {
		&s_failed($cmd_name, $wmode);
	}
	$cont;				# Continue status
}

# Each filter command is handled by a specific function. The Filter array
# maps an action name to a subroutine, while the Rfilter array lists the
# authorized actions in the special mode _SEEN_ (used when a mail already
# filtered is processed).
# The %Nostatus array records the commands which do not modify the execution
# status recorded by the last command. Typically, those are commands which can
# never fail.
sub init_filter {
	%Filter = (
		'ABORT', 'run_abort',		# Aborts application of filtering rules
		'ANNOTATE', 'run_annotate',	# Add new field into header
		'ASSIGN', 'run_assign',		# Assign value to variable
		'BACK', 'run_back',			# Eval feedback
		'BEGIN', 'run_begin',		# Enter in a new state
		'BOUNCE', 'run_bounce',		# Bounce message
		'DELETE', 'run_delete',		# Throw mail away, explicitely
		'FEED', 'run_feed',			# Feed back mail through program
		'FORWARD', 'run_forward',	# Forward mail
		'GIVE', 'run_give',			# Give body to command
		'KEEP', 'run_keep',			# Keep only the listed header fields
		'LEAVE', 'run_leave',		# Saving in incomming mailbox
		'MESSAGE', 'run_message',	# Send a vacation-like file
		'NOP', 'run_nop',			# No operation
		'NOTIFY', 'run_notify',		# Notify reception of message
		'ONCE', 'run_once',			# Once control
		'PASS', 'run_pass',			# Pass body to program with feedback
		'PERL', 'run_perl',			# Perform actions from within a perl script
		'PIPE', 'run_pipe',			# Pipe message to specified command
		'POST', 'run_post',			# Post mail to the net
		'PROCESS', 'run_process',	# Mailagent processing
		'PURIFY', 'run_purify',		# Purify header through a program
		'QUEUE', 'run_queue',		# Queue mail
		'RECORD', 'run_record',		# Record message in history
		'REJECT', 'run_reject',		# Reject
		'RESTART', 'run_restart',	# Restart
		'RESYNC', 'run_resync',		# Resynchronizes the header
		'RUN', 'run_run',			# Run specified program
		'SAVE', 'run_save',			# Save in a folder
		'SELECT', 'run_select',		# Time selection control
		'SPLIT', 'run_split',		# Split digest message
		'STORE', 'run_store',		# Save and leave copy in mailbox
		'STRIP', 'run_strip',		# Strip some header lines
		'SUBST', 'run_subst',		# Substitution on variable
		'TR', 'run_tr',				# Translation on variable
		'UNIQUE', 'run_unique',		# Delete message if already in history
		'VACATION', 'run_vacation',	# Allow or forbid vacation messages
		'WRITE', 'run_write',		# Write mail in folder
	);
	# Restricted filter actions: the commands listed below cannot be
	# executed in the special seen mode (in order to avoid loops).
	%Rfilter = (
		'BACK', 1,
		'BOUNCE', 1,
		'FEED', 1,
		'FORWARD', 1,
		'GIVE', 1,
		'NOTIFY', 1,
		'PASS', 1,
		'PIPE', 1,
		'POST', 1,
		'PURIFY', 1,
		'QUEUE', 1,
		'RUN', 1,
	);
	# The following commands do not modify the last status recorded.
	%Nostatus = (
		'ABORT', 1,
		'ASSIGN', 1,
		'BEGIN', 1,
		'KEEP', 1,
		'NOP', 1,
		'REJECT', 1,
		'RESTART', 1,
		'RESYNC', 1,
		'STRIP', 1,
		'VACATION', 1,
	);
}

